﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework.Input;

//I made this class for another game I'm working on, but it's not really
//anything special. Feel free to use it however you want in other projects.

namespace Betafy {
    public class KeyHelper {
        List<KeyWatcher> keyList;

        public delegate void KeyPressedHandler(KeyPressedEventArgs e);
        public event KeyPressedHandler KeyPressed;

        public KeyHelper() {
            keyList = new List<KeyWatcher>();
        }

        /// <summary>
        /// Adds a key to the watch list.
        /// </summary>
        /// <param name="key">Key to add.</param>
        public void WatchKey(ConsoleKey key) {
            keyList.Add(new KeyWatcher(key));
        }

        /// <summary>
        /// Clears all watched keys.
        /// </summary>
        public void ClearKeys() {
            keyList.Clear();
        }

        /// <summary>
        /// Removes any and all occurrences of the specified key.
        /// </summary>
        /// <param name="key">Key to remove.</param>
        public void RemoveKey(ConsoleKey key) {
            for (int i = 0; i < keyList.Count; i++)
                if (keyList[i].Key == key)
                    keyList.RemoveAt(i);
        }

        /// <summary>
        /// Updates all watched keys.
        /// </summary>
        public void Update() {
            KeyboardState keyState = Keyboard.GetState();
            for (int i = 0; i < keyList.Count; i++) {
                if (keyState.IsKeyDown((Keys)keyList[i].Key)) {
                    keyList[i].IsKeyUp = false;
                    keyList[i].IsKeyDown = true;
                }
                else {
                    keyList[i].IsKeyUp = true;
                }
                if (keyList[i].IsKeyUp == true && keyList[i].IsKeyDown == true) {
                    keyList[i].WasKeyPressed = true;
                    keyList[i].IsKeyDown = false;
                }

                if (keyList[i].WasKeyPressed) {
                    OnKeyPressed(keyList[i].Key);
                    resetKey(i);
                }
            }
        }

        /// <summary>
        /// Resets a key at the specified index.
        /// </summary>
        /// <param name="i">Index of the key.</param>
        void resetKey(int i) {
            keyList[i].WasKeyPressed = false;
            keyList[i].IsKeyDown = false;
            keyList[i].IsKeyUp = true;
        }

        /// <summary>
        /// Forces the KeyPressed event to trigger.
        /// </summary>
        /// <param name="key">The key to force.</param>
        public void OnKeyPressed(ConsoleKey key) {
            if (KeyPressed != null)
                KeyPressed(new KeyPressedEventArgs(key));
        }
        
        /// <summary>
        /// A class used to watch keys.
        /// </summary>
        class KeyWatcher {
            public ConsoleKey Key;
            public bool WasKeyPressed = false;
            public bool IsKeyDown = false;
            public bool IsKeyUp = true;
            public KeyWatcher(ConsoleKey k) { this.Key = k; }
        }
    }

    /// <summary>
    /// Event args specifically for a key being pressed.
    /// </summary>
    public class KeyPressedEventArgs : EventArgs {
        ConsoleKey key;
        public KeyPressedEventArgs(ConsoleKey key) {
            this.key = key;
        }

        public ConsoleKey PressedKey {
            get {
                return this.key;
            }
        }
    }
}