using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Betafy;

namespace Particles {
    /// <summary>
    /// This is mah particle test!
    /// </summary>
    public class ParticleTest : Microsoft.Xna.Framework.Game {
        static string picName = "PIC";
        int brushSize = 0;
        bool isPaused;
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Texture2D particle;
        CalcInterface calcInterface;
        List<Point> particleList = new List<Point>();
        Random rand = new Random();
        KeyHelper keyHelper;
        Mode mode;

        enum Mode {
            Sandbox, Editing
        }

        public ParticleTest() {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            this.IsMouseVisible = true;
            mode = Mode.Editing;
            keyHelper = new KeyHelper();
            keyHelper.WatchKey(ConsoleKey.Enter);
            keyHelper.WatchKey(ConsoleKey.Spacebar);
            keyHelper.KeyPressed += new KeyHelper.KeyPressedHandler(keyHelper_KeyPressed);
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize() {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent() {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            particle = this.Content.Load<Texture2D>("particle");            //Should be a square
            calcInterface = new CalcInterface(this, graphics, particle);    //Instantiate here after pixel texture is loaded
            calcInterface.ScaleFactor = 5;                                  //Should be width (and height) of pixel
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent() { }

        private void keyHelper_KeyPressed(KeyPressedEventArgs e) {
            if (e.PressedKey == ConsoleKey.Enter)
                if (mode == Mode.Sandbox) {
                    foreach (Point p in particleList)
                        calcInterface.PxlOff(p);
                    mode = Mode.Editing;
                }
                else if (mode == Mode.Editing) {
                    for (int i = 0; i < particleList.Count; i++)
                        if (calcInterface.PxlTest(particleList[i]))
                            particleList.RemoveAt(i);
                    mode = Mode.Sandbox;
                }
            if (e.PressedKey == ConsoleKey.Spacebar)
                isPaused = !isPaused;
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime) {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();
            keyHelper.Update();
            
            if (mode == Mode.Sandbox && !isPaused) {
                calcInterface.ClrScreen();
                calcInterface.RecallPic(picName);
                if (Mouse.GetState().LeftButton == ButtonState.Pressed) {
                    if (!calcInterface.PxlTest(getMousePoint()) && !particleList.Contains(getMousePoint()))
                        if (getMousePoint().X < calcInterface.ScreenWidth && getMousePoint().X > -1 &&
                            getMousePoint().Y < calcInterface.ScreenHeight && getMousePoint().Y > -1)
                            particleList.Add(getMousePoint());
                }

                for (int i = 0; i < particleList.Count; i++) {
                    Point p = particleList[i];
                    if (calcInterface.PxlTest(p.X, p.Y + 1)) {
                        rand = new Random(rand.Next());
                        if (!calcInterface.PxlTest(p.X - 1, p.Y) && !calcInterface.PxlTest(p.X + 1, p.Y))
                            p.X += (rand.Next(2) * 2) - 1;
                        else if (!calcInterface.PxlTest(p.X - 1, p.Y) && calcInterface.PxlTest(p.X + 1, p.Y))
                            p.X--;
                        else if (calcInterface.PxlTest(p.X - 1, p.Y) && !calcInterface.PxlTest(p.X + 1, p.Y))
                            p.X++;
                    }
                    else {
                        p.Y++;
                    }
                    particleList[i] = p;
                    calcInterface.PxlOn(p);
                }
                Cleanup();
            }
            else if (mode == Mode.Editing && !isPaused) {
                calcInterface.StorePic(picName);

                if (Mouse.GetState().LeftButton == ButtonState.Pressed)
                    calcInterface.DrawRectangle(getBrushRect(), false);
                else if (Mouse.GetState().RightButton == ButtonState.Pressed)
                    calcInterface.DrawRectangle(getBrushRect(), true);
            }

            UpdateBrush();
            if (!isPaused)
                Window.Title = "[Particles: " + particleList.Count.ToString() +
                           ", Mode: " + mode.ToString() +
                           ", Brush size: " + (brushSize + 1).ToString() + "]";
            else
                Window.Title = "[PAUSED]";
            base.Update(gameTime);
        }

        Point getMousePoint() {
            Point p = new Point(Mouse.GetState().X, Mouse.GetState().Y);

            p.X = (int)Math.Floor((float)p.X / calcInterface.ScaleFactor);
            p.Y = (int)Math.Floor((float)p.Y / calcInterface.ScaleFactor);
            
            return p;
        }

        Rectangle getBrushRect() {
            return new Rectangle(getMousePoint().X - (brushSize / 2),
                        getMousePoint().Y - (brushSize / 2), brushSize, brushSize);
        }

        /// <summary>
        /// Removes particles that are off the screen.
        /// </summary>
        void Cleanup() {
            for (int i = 0; i < particleList.Count; i++)
                if (particleList[i].Y > calcInterface.ScreenHeight)
                    particleList.RemoveAt(i);
        }

        void UpdateBrush() {
            if (mode == Mode.Editing)
                brushSize = Math.Abs(Mouse.GetState().ScrollWheelValue / 120);
            else brushSize = 0;
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime) {
            GraphicsDevice.Clear(Color.White);                      //Color is color you want the background to be
            spriteBatch.Begin();

            calcInterface.DrawScreen(spriteBatch, Color.Black);     //Color is the color you want pixels to be

            base.Draw(gameTime);
            spriteBatch.End();
        }
    }
}